@@ -80,8 +80,9 @@ for(fname in intersect(getGroupMembers("Math2"),
     if (!is.primitive(get(fname))) setGeneric(fname, group="Math2")
 
 
-## "Summary" --- this needs some hoop jumping that may become unnecessary
-##               in a future version of R (>= 2.3.x):
+## "Summary"
+if(!is.primitive(max)) { ## or another R 2.6.0 solution
+    ## --- some hoop jumping that is needed at least for R versions <= 2.5.x
 
 .max_def <- function(x, ..., na.rm = FALSE) base::max(x, ..., na.rm = na.rm)
 .min_def <- function(x, ..., na.rm = FALSE) base::min(x, ..., na.rm = na.rm)
@@ -106,6 +107,9 @@ setGeneric("any", function(x, ..., na.rm = FALSE) standardGeneric("any"),
 setGeneric("all", function(x, ..., na.rm = FALSE) standardGeneric("all"),
            useAsDefault = .all_def, group="Summary")
 
+} # end{hoop jumping}
+
+
 ## Add '...' so our methods can add  'sparseResult':
 setGeneric("colSums",
 	   def = function(x, na.rm = FALSE, dims = 1, ...) standardGeneric("colSums"),
