@@ -24,7 +24,7 @@ setAs("dtrMatrix", "CsparseMatrix", .dense2C)
 
 ## needed for t() method
 setAs("dtrMatrix", "matrix",
-      function(from) .Call(dtrMatrix_as_matrix, from))
+      function(from) .Call(dtrMatrix_as_matrix, from, TRUE))
 
 setAs("matrix", "dtrMatrix",
       function(from) as(.Call(dup_mMatrix_as_dgeMatrix, from), "dtrMatrix"))
@@ -51,7 +51,7 @@ setMethod("%*%", signature(x = "dtrMatrix", y = "matrix"),
 	  function(x, y) .Call(dtrMatrix_matrix_mm, x, y, FALSE),
           valueClass = "dgeMatrix")
 
-setMethod("%*%", signature(x = "dgeMatrix", y = "dtrMatrix"),
+setMethod("%*%", signature(x = "ddenseMatrix", y = "dtrMatrix"),
 	  function(x, y) .Call(dtrMatrix_matrix_mm, y, x, TRUE),
           valueClass = "dgeMatrix")
 
